#' @importFrom DOSE enrichDO
#' @export 
DOSE::enrichDO

#' @importFrom DOSE gseDO
#' @export 
DOSE::gseDO


##' @importFrom GOSemSim read.gaf
##' @export
GOSemSim::read.gaf

##' @importFrom GOSemSim read.blast2go
##' @export
GOSemSim::read.blast2go


##' @importFrom GOSemSim buildGOmap
##' @export
GOSemSim::buildGOmap

##' @importFrom magrittr %>%
##' @export
magrittr::`%>%`

##' @importFrom magrittr %<>%
##' @export
magrittr::`%<>%`

##' @importFrom gson read.gmt
##' @export
gson::read.gmt

##' @importFrom gson read.gmt.wp
##' @export
gson::read.gmt.wp


##' @importFrom dplyr arrange
##' @export
dplyr::arrange

##' @importFrom dplyr filter
##' @export
dplyr::filter

##' @importFrom dplyr group_by
##' @export
dplyr::group_by

##' @importFrom dplyr mutate
##' @export
dplyr::mutate

##' @importFrom dplyr n
##' @export
dplyr::n

##' @importFrom dplyr rename
##' @export
dplyr::rename

##' @importFrom dplyr select
##' @export
dplyr::select

##' @importFrom dplyr slice
##' @export
dplyr::slice

##' @importFrom dplyr summarise
##' @export
dplyr::summarise

