
#' Datasets
#' gcSample contains a sample of gene clusters.
#'
#'
#' @name DataSet
#' @aliases gcSample
#' @docType data
#' @keywords datasets
NULL

#' Datasets
#' kegg_species contains kegg species information
#'
#'
#' @name DataSet
#' @aliases kegg_species
#' @docType data
#' @keywords datasets
NULL

#' Datasets
#' kegg_category contains kegg pathway category information
#'
#'
#' @name DataSet
#' @aliases kegg_category
#' @docType data
#' @keywords datasets
NULL


#' Datasets
#' DE_GSE8057 contains differential epxressed genes obtained from GSE8057 dataset
#'
#'
#' @name DataSet
#' @aliases DE_GSE8057
#' @docType data
#' @keywords datasets
NULL


