## ##' plot method generics
## ##'
## ##'
## ##' @docType methods
## ##' @name plot
## ##' @rdname plot-methods
## ##' @title plot method
## ##' @param ... Additional argument list
## ##' @return plot
## ##' @importFrom stats4 plot
## ##' @export
## ##' @author Guangchuang Yu \url{https://yulab-smu.top}
## if ( !isGeneric("plot") )
## 	setGeneric("plot", function(x, ...) standardGeneric("plot"))


setGeneric("simplify", function(x, ...) standardGeneric("simplify"))



