% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/kegg-utilities.R
\name{ko2name}
\alias{ko2name}
\title{ko2name}
\usage{
ko2name(ko)
}
\arguments{
\item{ko}{ko ID}
}
\value{
data.frame
}
\description{
convert ko ID to descriptive name
}
\author{
guangchuang yu
}
