% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/bitr.R
\name{bitr}
\alias{bitr}
\title{bitr}
\usage{
bitr(geneID, fromType, toType, OrgDb, drop = TRUE)
}
\arguments{
\item{geneID}{input gene id}

\item{fromType}{input id type}

\item{toType}{output id type}

\item{OrgDb}{annotation db}

\item{drop}{drop NA or not}
}
\value{
data.frame
}
\description{
Biological Id TRanslator
}
\author{
Guangchuang Yu
}
